FuturesUnorderedIter

Struct FuturesUnorderedIter 

pub struct FuturesUnorderedIter<T, F>
where F: Future, T: Iterator<Item = F>,
{ /* private fields */ }
Bounded FuturesUnordered from an iterator.

This struct consumes an iterator where each item is a future. By using an iterator, we need no additional state, or additional queues to store the excess of futures.

This should be always preferred over FuturesUnorderedBounded, as it’s more performant and adds basically ZERO overhead on top of the normal FuturesUnordered.

Implementations

Source

impl<T, F> FuturesUnorderedIter<T, F>
where F: Future, T: Iterator<Item = F>,

pub fn new<I: IntoIterator<IntoIter = T>>( max_concurrent: usize, tasks: I, ) -> Self

Creates a new bounded FuturesUnordered from an iterator. This option is as efficient as it gets since it completely avoids allocating a new vector to store the excess of futures.

Since iterators are lazily evaluated, futures will be created on the fly as well.

Panics if max_concurrent is 0.

use futures_ratelimit::unordered::FuturesUnorderedIter;
use futures_ratelimit::common::Passthrough;
use futures::StreamExt;

async fn dummy() -> u64{
    42
}

let tasks = (0..100).into_iter().map(|_| dummy());
let mut fut_unordered = FuturesUnorderedIter::new(5, tasks);

// The internal FuturesUnorderedBounded will have 5 futures at most.
// The rest of futures will be pulled from the iterator as needed.

tokio_test::block_on(async move{
    assert_eq!(fut_unordered.borrow_inner().len(), 5);

    while let Some(value) = fut_unordered.next().await {
        println!("{}", value);
        assert!(fut_unordered.borrow_inner().len() <= 5);
    }

    assert_eq!(fut_unordered.borrow_inner().len(), 0);
    assert!(fut_unordered.borrow_inner().is_empty());

});